import React, { memo, useEffect } from 'react';
import { useLegacyState } from '../hooks/useLegacyState';
import { useInterpolater } from '../hooks/useInterpolater';

const ComponentWrapper = ({
  id,
  config,
  components,
  children,
  ...props
}) => {
  const {
    initState = {}
  } = config;
  const [state, setState] = useLegacyState(initState);
  const data = { ...props,
    state,
    setState
  };
  const {
    onMount,
    onUnmount,
    component
  } = useInterpolater(config, data);
  useEffect(() => {
    if (typeof onMount === 'function') {
      onMount();
    }

    if (typeof onUnmount === 'function') {
      return onUnmount;
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, []);

  if (typeof component === 'object' && component != null) {
    const Component = components[component.type];

    if (Component) {
      return React.createElement(Component, component.props, children);
    } else {
      const errorMsg = `${id}: component type ${component.type} not found.`;
      console.error(errorMsg);
      return null;
    }
  } else {
    console.error(`${id}: missing component field.`);
    return null;
  }
};

ComponentWrapper.defaultProps = {
  config: {},
  components: {}
};
export default memo(ComponentWrapper);